@@ -1,4 +1,5 @@
 from pydraw.errors import *;
+import math;
 
 
 class Location:
@@ -12,6 +13,9 @@ class Location:
                 location = (args[0][0], args[0][1]);
             elif len(args) == 2 and [type(arg) is float or type(arg) is int for arg in args]:
                 location = (args[0], args[1]);
+            else:
+                raise InvalidArgumentError('Location constructor takes a tuple/location '
+                                           'or two numbers (x, y)!');
         elif len(kwargs) == 0:
             raise InvalidArgumentError('Location constructor takes a tuple/location '
                                        'or two numbers (x, y)!');
@@ -48,6 +52,9 @@ class Location:
                 diff = (args[0][0], args[0][1]);
             elif len(args) == 2 and [type(arg) is float or type(arg) is int for arg in args]:
                 diff = (args[0], args[1]);
+            else:
+                raise InvalidArgumentError('move() takes a tuple/Location '
+                                           'or two numbers (dx, dy)!');
         elif len(kwargs) == 0:
             raise InvalidArgumentError('move() takes a tuple/Location '
                                        'or two numbers (dx, dy)!');
@@ -86,6 +93,9 @@ class Location:
                 location = (args[0][0], args[0][1]);
             elif len(args) == 2 and [type(arg) is float or type(arg) is int for arg in args]:
                 location = (args[0], args[1]);
+            else:
+                raise InvalidArgumentError('move() takes a tuple/Location '
+                                           'or two numbers (dx, dy)!');
         elif len(kwargs) == 0:
             raise InvalidArgumentError('moveto() takes a tuple/location '
                                        'or two numbers (dx, dy)!');
@@ -117,6 +127,15 @@ class Location:
 
         return self._y;
 
+    def distance(self, location) -> float:
+        """
+        Returns the distance between this location and another
+        :param location: the Location to get the distance to
+        :return: a float
+        """
+
+        return math.sqrt((location.x() - self.x()) ** 2 + (location.y() - self.y()) ** 2);
+
     def __str__(self):
         return f'(X: {self._x}, Y: {self._y})';
 
@@ -153,3 +172,6 @@ class Location:
             return False;
 
         return self.x() == other[0] and self.y() == other[1];
+
+    def __hash__(self):
+        return hash(self._x) ^ hash(self._y);
