@@ -117,6 +117,17 @@ class Object:
         """
         pass;
 
+    # noinspection PyProtectedMember
+    def _check(self) -> None:
+        if self._screen is None or Screen._TERMINATING:
+            return;  # We don't wanna mess with how stupid tk and turtle are.
+
+        if not self._screen.contains(self):
+            if self in self._screen._gridlines or self in self._screen._helpers:
+                return;
+
+            raise PydrawError('Cannot update or draw object that is not on the Screen!');
+
     def update(self) -> None:
         """
         To be overriden.
@@ -485,22 +496,44 @@ class Renderable(Object):
         if not isinstance(renderable, Renderable):
             raise TypeError('Passed non-renderable into Renderable#overlaps(), which takes only Renderables!');
 
-        min_ax = self.x();
-        max_ax = self.x() + self.width();
+        # Only optimize if the angle is not zero.
+        if self._angle == 0:
+            min_ax = self.x();
+            max_ax = self.x() + self.width();
 
-        min_bx = renderable.x();
-        max_bx = renderable.x() + renderable.width();
+            min_bx = renderable.x();
+            max_bx = renderable.x() + renderable.width();
 
-        min_ay = self.y();
-        max_ay = self.y() + self.height();
+            min_ay = self.y();
+            max_ay = self.y() + self.height();
 
-        min_by = renderable.y();
-        max_by = renderable.y() + renderable.height();
+            min_by = renderable.y();
+            max_by = renderable.y() + renderable.height();
 
-        a_left_b = max_ax < min_bx;
-        a_right_b = min_ax > max_bx;
-        a_above_b = min_ay > max_by;
-        a_below_b = max_ay < min_by;
+            a_left_b = max_ax < min_bx;
+            a_right_b = min_ax > max_bx;
+            a_above_b = min_ay > max_by;
+            a_below_b = max_ay < min_by;
+        else:
+            hypotenuse = math.sqrt(self.width() ** 2 + self.height() ** 2);
+            other_hypotenuse = math.sqrt(renderable.width() ** 2 + renderable.height() ** 2);
+
+            min_ax = self.x();
+            max_ax = self.x() + hypotenuse;
+
+            min_bx = renderable.x();
+            max_bx = renderable.x() + other_hypotenuse;
+
+            min_ay = self.y();
+            max_ay = self.y() + hypotenuse;
+
+            min_by = renderable.y();
+            max_by = renderable.y() + other_hypotenuse;
+
+            a_left_b = max_ax < min_bx;
+            a_right_b = min_ax > max_bx;
+            a_above_b = min_ay > max_by;
+            a_below_b = max_ay < min_by;
 
         # Do a base check to make sure they are even remotely near each other.
         if a_left_b or a_right_b or a_above_b or a_below_b:
@@ -690,6 +723,8 @@ class Renderable(Object):
         return new_vertices;
 
     def update(self):
+        self._check();
+
         old_ref = self._ref;
         shape = self._shape;  # List of normal vertices.
 
@@ -974,6 +1009,8 @@ class CustomPolygon(CustomRenderable):
         return new_vertices;
 
     def update(self):
+        self._check();
+
         old_ref = self._ref;
 
         xmin = self._vertices[0][0];
@@ -1211,6 +1248,8 @@ class Polygon(Renderable):
 
     # noinspection PyProtectedMember
     def update(self):
+        self._check();
+
         old_ref = self._ref;
         shape = self._shape;  # List of normal vertices.
 
@@ -1287,6 +1326,9 @@ class Image(Renderable):
                  visible: bool = True,
                  location: Location = None):
         self._image_name = image;
+        self._original = None;
+
+        # Filetype Checking
         split = image.split('.');
         if len(split) <= 1:
             raise PydrawError('File must have extension filetype:', self._image_name);
@@ -1303,6 +1345,8 @@ class Image(Renderable):
             try:
                 from PIL import Image, ImageTk;
                 image = Image.open(self._image_name);
+                self._original = image;  # We save the originally loaded image for easy modification
+
                 self._image = ImageTk.PhotoImage(image);
             except:
                 raise UnsupportedError('As PIL is not installed, only .png, .gif, and .ppm images are supported! '
@@ -1334,10 +1378,15 @@ class Image(Renderable):
         if border is not None:
             self.border(border);
 
+    # noinspection PyProtectedMember
     def _setup(self):
         # Pre-register the vertices so we don't have issues with .center()
         self._vertices = self.vertices();
 
+        real_location = self._screen.canvas_location(self.x(), self.y());
+        self._ref = self._screen._canvas.create_image(real_location.x() + self._width / 2,
+                                                      real_location.y() + self._height / 2, image=self._image);
+
     def width(self, width: float = None) -> float:
         """
         Get or set the width of the image (REQUIRES: PIL or Pillow)
@@ -1544,14 +1593,10 @@ class Image(Renderable):
 
         ImageTk.PhotoImage.__del__ = new_del
 
-    # noinspection PyProtectedMember
-    def _setup(self):
-        real_location = self._screen.canvas_location(self.x(), self.y());
-        self._ref = self._screen._canvas.create_image(real_location.x() + self._width / 2,
-                                                      real_location.y() + self._height / 2, image=self._image);
-
     # noinspection PyProtectedMember
     def update(self, updated: bool = False):
+        self._check();
+
         if updated:
             try:
                 from PIL import Image, ImageTk, ImageOps;
@@ -1560,15 +1605,23 @@ class Image(Renderable):
                     self._monkey_patch_del();  # If we do have PIL we need to monkey patch this immediately.
                     self._patched = True;
 
-                image = Image.open(self._image_name);
+                # Optimized (only read file once, caching everything else)
+                # TODO: In the future, caching images by filename could increase efficiency, but have serious pitfalls.
+                # ^ Perhaps if we hash the image we could then compare against future hashes to check if the file
+                # has been modified or not. (Noah)
+                if self._original is not None:
+                    image = self._original.copy();
+                else:
+                    image = Image.open(self._image_name);
+                    self._original = image.copy();
 
                 if self._frame != -1:
                     try:
                         image.seek(self._frame);
                     except EOFError:
-                        raise PydrawError('No more images in GIF File!');
+                        raise PydrawError(f'No more frames in GIF: {self._image_name}!');
 
-                image = image.convert('RGBA');
+                image = image.convert('RGBA');  # Convert so we can color-filter the image
 
                 if self._color is not None and self._color != Color.NONE:
                     r, g, b, alpha = image.split()
@@ -1953,6 +2006,7 @@ class Text(CustomRenderable):
 
     # noinspection PyProtectedMember
     def update(self) -> None:
+        self._check();
         # super().update(); | JUST FOR RENDERABLES - DO NOT USE
         # we are going to delete and re-add text to the screen. You cannot alter a text object.
         old_ref = self._ref;
@@ -2484,6 +2538,8 @@ class Line(Object):
 
     # noinspection PyProtectedMember
     def update(self):
+        self._check();
+
         try:
             old_ref = self._ref;
 
