@@ -114,7 +114,11 @@ class Screen:
     the window that is created. Sort of like a canvas.
     """
 
-    def __init__(self, width=800, height=600, title="pydraw"):
+    _TERMINATING = False
+
+    def __init__(self, width: int = 800, height: int = 600, title: str = "pydraw"):
+        verify(width, int, height, int, title, str);
+
         self._screen = turtle.Screen();
         self._turtle = turtle;
         self._canvas = self._screen.cv;
@@ -156,12 +160,20 @@ class Screen:
         self._screen.tracer(0);
         self._screen.update();
 
+        self._scene = None;  # We store our current Scene.
+
         # import atexit;
         # self._root.protocol('WM_DELETE_WINDOW', self._exit_handler);
         # atexit.register(self._exit_handler)
 
         # --- #
 
+        def onclose():
+            Screen._TERMINATING = True
+            self._root.destroy();
+
+        self._root.protocol("WM_DELETE_WINDOW", onclose);
+
         self.registry = {};  # The input function registry (stores input callbacks)
 
     def title(self, title: str = None) -> str:
@@ -172,6 +184,7 @@ class Screen:
         """
 
         if title is not None:
+            verify(title, str);
             self._title = title;
             self._screen.title(title);
 
@@ -185,6 +198,7 @@ class Screen:
         """
 
         if color is not None:
+            verify(color, Color);
             self._color = color;
             self._screen.bgcolor(color.__value__());
         return self._color;
@@ -196,6 +210,7 @@ class Screen:
         :return: None
         """
 
+        verify(pic, str);
         self._screen.bgpic(pic);
 
     def resize(self, width: int, height: int) -> None:
@@ -206,6 +221,7 @@ class Screen:
         :return: None
         """
 
+        verify(width, int, height, int);
         # noinspection PyBroadException
         try:
             self._screen.screensize(width, height);
@@ -253,6 +269,7 @@ class Screen:
         """
         Gets the center of the screen.
         """
+
         return Location(self.width() / 2, self.height() / 2);
 
     # noinspection PyMethodMayBeStatic
@@ -308,6 +325,8 @@ class Screen:
         """
         from tkinter.simpledialog import SimpleDialog
 
+        verify(text, str, title, str, accept_text, str, cancel_text, str);
+
         alert = SimpleDialog(self._root,
                              text=text,
                              buttons=[accept_text, cancel_text],
@@ -324,6 +343,8 @@ class Screen:
         :return: None
         """
 
+        verify(text, str, title, str);
+
         text = self._screen.textinput(title, text);
 
         self._screen.listen();  # keep us nice and listening :)
@@ -332,6 +353,8 @@ class Screen:
     def grid(self, rows: int = None, cols: int = None, cellsize: tuple = (50, 50), helpers: bool = True):
         from pydraw import Line, Text;
 
+        verify(rows, int, cols, int, cellsize, tuple, helpers, bool);
+
         if len(self._gridlines) > 0:
             [line.remove() for line in self._gridlines];
             self._gridlines.clear();
@@ -424,6 +447,14 @@ class Screen:
         :return: the name of the file.
         """
 
+        if filename is None:
+            filename = 'pydraw' + str(time.time() % 10000);
+
+        verify(filename, str);
+
+        if not filename.endswith('.png'):
+            filename += '.png';
+
         # noinspection PyBroadException
         try:
             from PIL import ImageGrab;
@@ -434,12 +465,6 @@ class Screen:
             x2 = x1 + self.width() - BORDER_CONSTANT;
             y2 = y1 + self.height() - BORDER_CONSTANT;
 
-            if filename is None:
-                filename = 'pydraw' + str(time.time() % 10000);
-
-            if not filename.endswith('.png'):
-                filename += '.png';
-
             ImageGrab.grab().crop((x1, y1, x2, y2)).save(filename);
             return filename;
         except:
@@ -457,6 +482,7 @@ class Screen:
         """
 
         if fullscreen is not None:
+            verify(fullscreen, bool);
             self._fullscreen = fullscreen;
             self._root.attributes("-fullscreen", fullscreen);
             self.update();
@@ -464,6 +490,10 @@ class Screen:
         return self._fullscreen;
 
     def _front(self, obj) -> None:
+        from pydraw import Object;
+
+        if not isinstance(obj, Object):
+            raise InvalidArgumentError(f'Expected an Objcet {obj}, instead got {type(obj)}.');
 
         self._canvas.tag_raise(obj._ref);
 
@@ -471,7 +501,8 @@ class Screen:
         from pydraw import Object;
 
         if not isinstance(obj, Object):
-            raise InvalidArgumentError('Expected an Object...')
+            raise InvalidArgumentError(f'Expected an Objcet {obj}, instead got {type(obj)}.');
+
         self._canvas.tag_lower(obj._ref);
 
     def _add(self, obj) -> None:
@@ -483,6 +514,18 @@ class Screen:
 
         self._objects.append(obj);
 
+    def add(self, obj) -> None:
+        """
+        Add an object back to the Screen after having removed it (with Object.remove() or Screen.remove(object)
+        :param obj: the Object to add back.
+        :return: None
+        """
+
+        if obj in self._objects:
+            raise PydrawError(f'Cannot re-add object that is already in the object cache! {obj}: {type(obj)}');
+
+        self._add(obj);
+
     # noinspection PyProtectedMember
     def remove(self, obj):
         # self._screen.cv.delete(obj._ref);
@@ -504,6 +547,18 @@ class Screen:
 
         return tuple(self._objects);
 
+    def contains(self, obj) -> bool:
+        """
+        Returns whether or not the passed object exists on the Screen (is in the objects cache)
+        :param obj: the Object to check
+        :return: a boolean
+        """
+
+        return obj in self._objects;
+
+    def __contains__(self, item):
+        return self.contains(item);
+
     def clear(self) -> None:
         """
         Clears the screen.
@@ -513,13 +568,13 @@ class Screen:
         try:
             for i in range(len(self._objects) - 1, -1, -1):
                 self._objects[i].remove();
-            if self._gridstate:
-                self._redraw_grid();  # Redraw the grid if it was active.
+            # if self._gridstate:
+            #     self._redraw_grid();  # Redraw the grid if it was active.
             self.color(self._color);  # Redraw the color of the screen.
         except (tk.TclError, AttributeError):
             pass;
 
-    def scene(self, scene) -> None:
+    def scene(self, scene=None):
         """
         Apply a new scene to the screen!
 
@@ -527,17 +582,29 @@ class Screen:
         :param scene: The Scene to apply!
         :return: None
         """
+        from pydraw import Scene;
 
-        self.reset();
-        scene.activate(self);
+        if not isinstance(scene, Scene):
+            raise InvalidArgumentError('You must pass a an object that extends Scene!');
+
+        if scene is None:
+            return self._scene;
+
+        if self._scene is not None:
+            del self._scene;
+
+        self.reset();  # Clears screen and destroys all registered input handlers.
 
         # Defines all input methods from the Scene.
-        for (name, function) in inspect.getmembers(scene, inspect.isfunction):
+        for (name, function) in inspect.getmembers(scene, predicate=inspect.ismethod):
             if name.lower() not in INPUT_TYPES:
                 continue;
 
             self.registry[name.lower()] = function;
 
+        self._scene = scene;
+        scene.activate(self);
+
     def reset(self) -> None:
         """
         Resets the screen, removing all objects and input methods.
@@ -546,6 +613,8 @@ class Screen:
 
         self.toggle_grid(False);
         self._gridlines.clear();
+        for line in self._gridlines:
+            line.remove();
 
         for obj in self._helpers:
             obj.remove();
